﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
//using System.Linq;
using System.Text;
using System.Windows.Forms;

using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.EditorInput;

namespace lufs_genplan
{
    public partial class FormLen : Form
    {
        public FormLen()
        {
            InitializeComponent();
            selected_color = null;
            CheckbTableUse.Checked = false;
            this.Size = new Size(602, 280);
        }
        public Autodesk.AutoCAD.Colors.Color selected_color;

        private void BtCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void CheckbLayerUse_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckbLayerUse.Checked) { CombobLayerUse.Enabled = true; CombobLayerUse.Text = (string)CombobLayerUse.Items[0]; }
            else { CombobLayerUse.Enabled = false; }
        }

        private void BtSelectColor_Click(object sender, EventArgs e)
        {
            Autodesk.AutoCAD.Windows.ColorDialog cd = new Autodesk.AutoCAD.Windows.ColorDialog();
            cd.IncludeByBlockByLayer = true;
            cd.SetDialogTabs(Autodesk.AutoCAD.Windows.ColorDialog.ColorTabs.ACITab);
            DialogResult resd=cd.ShowDialog();
            if ((resd == DialogResult.Cancel) && (selected_color == null)) { CheckbColorUse.CheckState = CheckState.Unchecked; return; }
            else if (resd == DialogResult.Cancel) { return; }
            selected_color=cd.Color;            
            pColor.BackColor = cd.Color.ColorValue;
            LbColorName.Text = "Цвет: "+cd.Color.ColorNameForDisplay;
            LbColorName.Update();
        }

        private void CheckbColorUse_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckbColorUse.Checked) 
            {
                BtSelectColor.Enabled = true;
                if (selected_color == null) { BtSelectColor_Click(sender, e); }
            }
            else { BtSelectColor.Enabled = false; }
        }

        private void CheckbLinetUse_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckbLinetUse.Checked) { CombobLinetUse.Enabled = true; CombobLinetUse.Text = (string)CombobLinetUse.Items[0]; }
            else { CombobLinetUse.Enabled = false; }
        }

        private void BtSolut_Click(object sender, EventArgs e)
        {
            if (RbUser.Checked)
            {
                Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
                Autodesk.AutoCAD.EditorInput.PromptSelectionOptions edOpt = new Autodesk.AutoCAD.EditorInput.PromptSelectionOptions();
                Autodesk.AutoCAD.EditorInput.PromptSelectionResult edRes;
                edOpt.MessageForAdding="Выберите объекты";
                edOpt.AllowDuplicates = false;
                Autodesk.AutoCAD.DatabaseServices.TypedValue[] selval;
                selval = GetFilteredParam();
                Autodesk.AutoCAD.EditorInput.SelectionFilter ssfilter = new Autodesk.AutoCAD.EditorInput.SelectionFilter(selval);
                edRes=ed.GetSelection(edOpt, ssfilter);
                
                SelectionSet sset=edRes.Value;
                if (edRes.Status == PromptStatus.OK) 
                {
                    double[] res=CalcRes(sset);
                    WriteResultToCommandPrompt(res);
                }
            }
            else if (RbAll.Checked)
            {
                Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
                Autodesk.AutoCAD.EditorInput.PromptSelectionResult edRes;

                Autodesk.AutoCAD.DatabaseServices.TypedValue[] selval;
            
                selval = GetFilteredParam();
                Autodesk.AutoCAD.EditorInput.SelectionFilter ssfilter = new Autodesk.AutoCAD.EditorInput.SelectionFilter(selval);

                edRes=ed.SelectAll(ssfilter);
                SelectionSet sset = edRes.Value;
                if (edRes.Status == PromptStatus.OK)
                {
                    double[] res = CalcRes(sset);
                    WriteResultToCommandPrompt(res);
                }
            }
        }

        //Возвращает массив параметров фильтра указанные пользователем в Комбобоксах
        private Autodesk.AutoCAD.DatabaseServices.TypedValue[] GetFilteredParam()
        {
            List<Autodesk.AutoCAD.DatabaseServices.TypedValue> selval_list = new List<Autodesk.AutoCAD.DatabaseServices.TypedValue>();
            //Фильтр по типу - полилиния
            if (RbPlineEnt.Checked)
            {
                selval_list.Add(new Autodesk.AutoCAD.DatabaseServices.TypedValue((int)(DxfCode.Start), "LWPOLYLINE"));
            }
            else
            {
                //Select all entitys
            }
            if (CheckbLayerUse.Checked && CombobLayerUse.Text != "")
            {
                //фильтр по слою - OK
                selval_list.Add(new Autodesk.AutoCAD.DatabaseServices.TypedValue((int)(DxfCode.LayerName), CombobLayerUse.Text));
            }
            if (CheckbColorUse.Checked)
            {
                //фильтр по цвету - OK - ОГРАНИЧЕНИЕ: применимы цвета из индексной палитры
                selval_list.Add(new Autodesk.AutoCAD.DatabaseServices.TypedValue((int)(DxfCode.Color), selected_color.ColorIndex));
            }
            if (CheckbLinetUse.Checked)
            {
                //фильтр по типулинии - ОК
                selval_list.Add(new Autodesk.AutoCAD.DatabaseServices.TypedValue((int)(DxfCode.LinetypeName), CombobLinetUse.Text));
            }
            return selval_list.ToArray();
        }

        //Выводит в командную строку значение длины (res[0]) и площади (res[1]) и количества обработанных объектов (res[2]) с пояснениями и коментариями
        private void WriteResultToCommandPrompt(double[] res)
        {
            Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            if (CheckbMasUse.Checked == false) 
            { ed.WriteMessage("\nВыделено " + res[2].ToString() + 
                " объектов.   " + "Длина " + res[0].ToString() + " мм чертежа.\nПлощадь " + res[1].ToString() + 
                " мм.кв. чертежа\n"); }
            else
            {
                ed.WriteMessage("\nВыделено " + res[2].ToString() + " объектов.   " +  
                    "Длина " + (LUFS_Utils.mashtToDouble(CombobMasht.Text) * res[0]).ToString() + 
                    " м в масштабе " + CombobMasht.Text + ".\n");
                ed.WriteMessage("Площадь " + (LUFS_Utils.mashtToDouble(CombobMasht.Text) * LUFS_Utils.mashtToDouble(CombobMasht.Text) * 
                    res[1]).ToString() + " м.кв. в масштабе " + CombobMasht.Text + ".\n");
            }
        }

        //Возвращает массив [0]=length, [1]=area, [2]=number of objects;
        private double[] CalcRes(SelectionSet sset)
        {
            Database db = HostApplicationServices.WorkingDatabase;
            Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                double length = 0;
                double area = 0;
                int count = 0;
                foreach (ObjectId id in sset.GetObjectIds())//
                {

                    BlockTableRecord BTR = (BlockTableRecord)trans.GetObject(
                                              trans.GetObject(id, OpenMode.ForRead).OwnerId,
                                              OpenMode.ForRead);
                    //ed.WriteMessage("\n -> " + (BTR.Name).ToUpper());
                    string BTR_name = (BTR.Name).ToUpper();
                    if (CheckbOnlyModel.Checked && BTR_name != "*MODEL_SPACE") { continue; };

                    if (RbPlineEnt.Checked)
                    {
                        Polyline pline;
                        pline = (Polyline)trans.GetObject(id, OpenMode.ForRead);
                        length += pline.Length;
                        area += pline.Area;
                        count++;
                    }
                    else 
                    {
                        DBObject selOb;
                        selOb = trans.GetObject(id, OpenMode.ForRead);
                        if (selOb.GetType().Name == "Polyline")
                        {
                            //ed.WriteMessage(selOb.GetType().Name + " -> ");
                            length += ((Polyline)selOb).Length;
                            area += ((Polyline)selOb).Area;
                            count++;
                        }else if(selOb.GetType().Name == "Line")
                        {
                            //ed.WriteMessage(selOb.GetType().Name + " -> ");
                            length += ((Line)selOb).Length;
                            //area += ((Line)selOb).Area;
                            count++;
                        }else if(selOb.GetType().Name == "Arc")
                        {
                            //ed.WriteMessage(selOb.GetType().Name + " -> ");
                            length += ((Arc)selOb).Length;
                            area += ((Arc)selOb).Area;
                            count++;
                        }else if (selOb.GetType().Name == "Circle")
                        {
                            //ed.WriteMessage(selOb.GetType().Name + " -> ");
                            length += ((Circle)selOb).Circumference;
                            area += ((Circle)selOb).Area;
                            count++;
                        }                       
                    }
                }
                double[] result = { length, area, count };
                return result;
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("BtSolut_Click() " + ex.ToString());
            }
            finally
            {
                trans.Commit();
                trans.Dispose();
                ed = null;
                db = null;
            }
            double[] result_error = { -1, -1, -1 };
            return result_error;
        }

        private void CheckbMasUse_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckbMasUse.Checked) { CombobMasht.Enabled = true;}
            else { CombobMasht.Enabled = false; }
        }

        private void btSample_Click(object sender, EventArgs e)
        {
            Autodesk.AutoCAD.EditorInput.Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.EditorInput.PromptEntityOptions edOpt = new Autodesk.AutoCAD.EditorInput.PromptEntityOptions("Выберите объект аналог\n");
            Autodesk.AutoCAD.EditorInput.PromptEntityResult edRes;
            if (RbAllEnt.Checked)
            {
                edOpt.SetRejectMessage("Объект не Arc,Circle,Line,Polyline.\n");
                edOpt.AddAllowedClass(typeof(Circle), false);
                edOpt.AddAllowedClass(typeof(Line), false);
                edOpt.AddAllowedClass(typeof(Polyline), false);
                edOpt.AddAllowedClass(typeof(Arc), false);
            }
            else
            {
                edOpt.SetRejectMessage("Объект не полилиния.\n");
                edOpt.AddAllowedClass(typeof(Polyline), false);
            }
            edOpt.AllowNone = false;
            edOpt.AllowObjectOnLockedLayer = false;

            edRes=ed.GetEntity(edOpt);
            if (edRes.Status == PromptStatus.OK)
            {
                Database db = HostApplicationServices.WorkingDatabase;
                Transaction trans = db.TransactionManager.StartTransaction();
                try
                {
                    Curve selCurve;
                    selCurve = (Curve)trans.GetObject(edRes.ObjectId, OpenMode.ForRead);
                    CheckbLayerUse.Checked = true;
                    CombobLayerUse.Text = selCurve.Layer;

                    selected_color = selCurve.Color;
                    CheckbColorUse.Checked = true;
                    pColor.BackColor = selCurve.Color.ColorValue;
                    LbColorName.Text = "Цвет: " + selCurve.Color.ColorNameForDisplay;
                    LbColorName.Update();

                    CheckbLinetUse.Checked = true;
                    CombobLinetUse.Text = selCurve.Linetype;

                }
                catch (System.Exception ex)
                {
                    ed.WriteMessage("btSample_Click() " + ex.ToString());
                }
                finally
                {
                    trans.Commit();
                    trans.Dispose();
                }
            }
            else { return; }
        }

        private void btAdd_Click(object sender, EventArgs e)
        {
            Param param = new Param(null,null,null,null,lufs_genplan.Param.calctype.all);
            if (CheckbLayerUse.Checked && CombobLayerUse.Text != "")
            {
                param.Layer = CombobLayerUse.Text;
            }
            else { param.Layer = "Любой"; }
            if (CheckbColorUse.Checked)
            {
                param.Color=selected_color;
            }
            else { param.Color = null; }
            if (CheckbLinetUse.Checked)
            {
                param.Linetype=CombobLinetUse.Text;
            }
            else { param.Linetype="Любой";}
            foreach (Param it in listBox1.Items)
            {
                if (it.Equal(param)) 
                { 
                    MessageBox.Show("Набор параметров уже существует (" + it.Description + ")", "Ошибка");
                    param = null;
                    return; 
                }
            }
            QueryBox qbox = new QueryBox();
            if (qbox.ShowQuery("Введите описание параметров:", "") == DialogResult.OK)
            {
                param.Description = qbox.GetText(); qbox.Dispose();
            }
            else { qbox.Dispose(); return; }

            listBox1.DisplayMember = "DisplayMember";
            listBox1.Items.Add(param);
        }

        private void CheckbTableUse_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckbTableUse.Checked) { groupBoxTable.Enabled = true; this.Size = new Size(602,500); }
            else { groupBoxTable.Enabled = false; this.Size = new Size(602,280); }
        }

        private void btAddDescr_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedItem != null)
            {
                QueryBox qbox = new QueryBox();
                if (qbox.ShowQuery("Введите описание параметров:", ((Param)listBox1.SelectedItem).Description) == DialogResult.OK)
                {

                    Param temp = (Param)listBox1.SelectedItem;
                    temp.Description = qbox.GetText();
                    qbox.Dispose();
                    int pos = listBox1.Items.IndexOf(listBox1.SelectedItem);
                    listBox1.Items.RemoveAt(pos);
                    listBox1.Items.Insert(pos, temp);
                }
                else { return; }
            }
            else { MessageBox.Show("Выберите элемент в списке!", "Внимание!"); };
        }

        private void btDel_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedItem != null)
            {
                int pos = listBox1.Items.IndexOf(listBox1.SelectedItem);
                listBox1.Items.RemoveAt(pos);
            }
        }
       
        private void btSave_Click(object sender, EventArgs e)
        {
            if (listBox1.Items.Count > 0)
            {
                QueryBox qbox = new QueryBox();
                if (qbox.ShowQuery("Введите имя таблицы:", "Ведомость покрытий") == DialogResult.OK)
                {
                    List<Param> res = new List<Param>();
                    foreach(Param it in listBox1.Items)
                    { res.Add(it); }
                    string table_name = qbox.GetText();
                    if (table_name == "") { table_name = "Ведомость покрытий"; }
                    
                    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
                    PromptPointResult pt_res = ed.GetPoint("Укажите точку вставки таблицы (верхний левый угол) ");
                    CreateTable(res, table_name, pt_res.Value.X, pt_res.Value.Y);
                }
            }
            else { return; }
        }

        //This function creates a new Table object,
        //and adds it to ModelSpace.
        private void CreateTable(List<Param> res, string table_name, double X, double Y)
        {
            Database db = HostApplicationServices.WorkingDatabase;
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                ObjectId layerId=CreateNewLayer(table_name);
                //***************************************************
                //Create header
                //***************************************************
                Point3dCollection vertex=new Point3dCollection();
                vertex.Add(new Point3d(X, Y, 0.0d));
                vertex.Add(new Point3d(X + 185.0d, Y, 0.0d));
                vertex.Add(new Point3d(X + 185.0d, Y - 20.0d, 0.0d));
                vertex.Add(new Point3d(X, Y - 20.0d, 0.0d));
                CreatePLine(vertex, layerId, 1.0d, true);
                CreateMText(X + 185.0d / 2.0d, Y + 4.0d, layerId, table_name, AttachmentPoint.MiddleCenter, 185.0d, 7.0d, 5.0d);

                vertex.Clear();
                vertex.Add(new Point3d(X+10.0d, Y, 0.0d));
                vertex.Add(new Point3d(X + 10.0d, Y-20.0d, 0.0d));
                CreatePLine(vertex, layerId, 1.0d, false);
                CreateMText(X + 5.0d, Y - 10.0d, layerId, "Поз.", AttachmentPoint.MiddleCenter, 10.0d, 20.0d, 2.5d);

                vertex.Clear();
                vertex.Add(new Point3d(X + 115.0d, Y, 0.0d));
                vertex.Add(new Point3d(X + 115.0d, Y - 20.0d, 0.0d));
                CreatePLine(vertex, layerId, 1.0d, false);
                CreateMText(X + 10.0d+105.0d/2.0d, Y - 10.0d, layerId, "Наименование", AttachmentPoint.MiddleCenter, 105.0d, 20.0d, 2.5d);

                vertex.Clear();
                vertex.Add(new Point3d(X + 125.0d, Y, 0.0d));
                vertex.Add(new Point3d(X + 125.0d, Y - 20.0d, 0.0d));
                CreatePLine(vertex, layerId, 1.0d, false);
                CreateMText(X + 120.0d, Y - 10.0d, layerId, "Тип", AttachmentPoint.MiddleCenter, 10.0d, 20.0d, 2.5d);

                vertex.Clear();
                vertex.Add(new Point3d(X + 155.0d, Y, 0.0d));
                vertex.Add(new Point3d(X + 155.0d, Y - 20.0d, 0.0d));
                CreatePLine(vertex, layerId, 1.0d, false);
                CreateMText(X + 140.0d, Y - 10.0d, layerId, "Площадь / длина", AttachmentPoint.MiddleCenter, 30.0d, 20.0d, 2.5d);

                CreateMText(X + 170.0d, Y - 10.0d, layerId, "Примечание", AttachmentPoint.MiddleCenter, 30.0d, 20.0d, 2.5d);

                //***************************************************
                //Create data lines
                //***************************************************
                int count = 0;
                foreach (Param datal in res)
                {
                    count++;
                    vertex.Clear();
                    vertex.Add(new Point3d(X, Y-20.0d-16.0d*((double)(count-1)), 0.0d));
                    vertex.Add(new Point3d(X, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 1.0d, false);
                    vertex.Clear();
                    vertex.Add(new Point3d(X+10.0d, Y - 20.0d - 16.0d * ((double)(count - 1)), 0.0d));
                    vertex.Add(new Point3d(X+10.0d, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 0.5d, false);
                    vertex.Clear();
                    vertex.Add(new Point3d(X + 115.0d, Y - 20.0d - 16.0d * ((double)(count - 1)), 0.0d));
                    vertex.Add(new Point3d(X + 115.0d, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 0.5d, false);
                    vertex.Clear();
                    vertex.Add(new Point3d(X + 125.0d, Y - 20.0d - 16.0d * ((double)(count - 1)), 0.0d));
                    vertex.Add(new Point3d(X + 125.0d, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 0.5d, false);
                    vertex.Clear();
                    vertex.Add(new Point3d(X + 155.0d, Y - 20.0d - 16.0d * ((double)(count - 1)), 0.0d));
                    vertex.Add(new Point3d(X + 155.0d, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 0.5d, false);
                    vertex.Clear();
                    vertex.Add(new Point3d(X + 185.0d, Y - 20.0d - 16.0d * ((double)(count - 1)), 0.0d));
                    vertex.Add(new Point3d(X + 185.0d, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 1.0d, false);
                    vertex.Clear();
                    vertex.Add(new Point3d(X, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    vertex.Add(new Point3d(X + 185.0d, Y - 20.0d - 16.0d * ((double)(count)), 0.0d));
                    CreatePLine(vertex, layerId, 0.5d, false);

                    CreateMText(X + 5.0d, Y - 20.0d - 8.0d - 16.0d * ((double)(count - 1)), layerId, count.ToString(), AttachmentPoint.MiddleCenter, 10.0d, 16.0d, 3.5d);
                    CreateMText(X + 10.0d + 2.0d, Y - 20.0d - 8.0d - 16.0d * ((double)(count - 1)), layerId, datal.Description, AttachmentPoint.MiddleLeft, 101.0d, 16.0d, 2.5d);
                    CreateMText(X + 120.0d, Y - 20.0d - 8.0d - 16.0d * ((double)(count - 1)), layerId, "~", AttachmentPoint.MiddleCenter, 10.0d, 16.0d, 2.5d);
                    CreateMText(X + 140.0d, Y - 20.0d - 8.0d - 16.0d * ((double)(count - 1)), layerId, "Length/Area", AttachmentPoint.MiddleCenter, 30.0d, 16.0d, 2.5d);
                    CreateMText(X + 170.0d, Y - 20.0d - 8.0d - 16.0d * ((double)(count - 1)), layerId, "~", AttachmentPoint.MiddleCenter, 30.0d, 16.0d, 2.5d);

                }

                trans.Commit(); 
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("Error Creating Table: " + ex.ToString());
            }
            finally
            {
                trans.Dispose(); // Manual cleanup necessary in this transaction model
            }
        }
        //Create Line
        ObjectId CreateLine(double x1, double y1, double x2, double y2, ObjectId layer, double thickness)
        {
            Point3d startpt = new Point3d(x1, y1, 0.0);
            Point3d endpt = new Point3d(x2, y2, 0.0);
            Line pLine = new Line(startpt, endpt);
            pLine.LayerId = layer;
            pLine.Thickness = thickness;

            ObjectId lineid;
            Database db = HostApplicationServices.WorkingDatabase;
            Transaction trans = db.TransactionManager.StartTransaction();
            try{
                BlockTable bt = (BlockTable)trans.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)trans.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                lineid = btr.AppendEntity(pLine);
                trans.AddNewlyCreatedDBObject(pLine, true);
                trans.Commit();
            }
            finally{
                trans.Dispose();
            }
            return lineid;
        }
        //Create Polyline2d
        ObjectId CreatePLine(Point3dCollection pt_collection, ObjectId layer, double wigth, bool closed)
        {
            DoubleCollection bulges = new DoubleCollection(pt_collection.Count);
            for (int i = 0; i < pt_collection.Count;  bulges.Add(0.0d), i++);

            Polyline2d pLine = new Polyline2d(Poly2dType.SimplePoly, pt_collection, 0.0d, closed, wigth, wigth, bulges);
            pLine.LayerId = layer;
                        
            ObjectId lineid;
            Database db = HostApplicationServices.WorkingDatabase;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                BlockTable bt = (BlockTable)trans.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)trans.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                lineid = btr.AppendEntity(pLine);
                trans.AddNewlyCreatedDBObject(pLine, true);
                trans.Commit();
            }
            finally
            {
                trans.Dispose();
            }
            return lineid;
        }

        ObjectId CreateMText(double x, double y, ObjectId layer, string text, AttachmentPoint attach, double width, double height, double font_height)
        {

            MText pText = new MText();
            pText.LayerId = layer;
            pText.Attachment = attach;
            pText.Location = new Point3d(x, y, 0.0d);
            pText.Width = width;
            pText.TextHeight = font_height;
            ObjectId styleId = GetStyleId("ISO");
            if (styleId != new ObjectId())
            {
                pText.TextStyleId = styleId;
            }//else use default textsyle STANDARD
            pText.Height = height;
            pText.Contents = text;

            ObjectId Textid;
            Database db = HostApplicationServices.WorkingDatabase;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                BlockTable bt = (BlockTable)trans.GetObject(db.BlockTableId, OpenMode.ForRead, false);
                BlockTableRecord btr = (BlockTableRecord)trans.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);
                Textid = btr.AppendEntity(pText);
                trans.AddNewlyCreatedDBObject(pText, true);
                trans.Commit();
            }
            finally
            {
                trans.Dispose();
            }
            return Textid;
        }

        ObjectId GetStyleId(string style_name)
        {
            ObjectId styleId = new ObjectId();
            Database db = HostApplicationServices.WorkingDatabase;
            Transaction trans = db.TransactionManager.StartTransaction();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            try
            {
                Autodesk.AutoCAD.DatabaseServices.TextStyleTable styleTable;
                styleTable = (Autodesk.AutoCAD.DatabaseServices.TextStyleTable)trans.GetObject(db.TextStyleTableId, OpenMode.ForRead, true);
                foreach (ObjectId str in styleTable)
                {
                    Autodesk.AutoCAD.DatabaseServices.TextStyleTableRecord sttr = (TextStyleTableRecord)trans.GetObject(str, OpenMode.ForRead, true);
                    if (sttr.Name == style_name) { return str; }
                }
                trans.Commit();
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("CreateNewLayer() " + ex.ToString());
            }
            finally
            {
                trans.Dispose();
            }

            return styleId;
        }

        bool isLayerExists(string layer_name)
        {
            Database db = HostApplicationServices.WorkingDatabase;
            Transaction trans = db.TransactionManager.StartTransaction();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            try
            {
                Autodesk.AutoCAD.DatabaseServices.LayerTable layerTable;
                layerTable = (Autodesk.AutoCAD.DatabaseServices.LayerTable)trans.GetObject(db.LayerTableId, OpenMode.ForRead, true);
                foreach (ObjectId str in layerTable)
                {
                    Autodesk.AutoCAD.DatabaseServices.LayerTableRecord ltr = (LayerTableRecord)trans.GetObject(str, OpenMode.ForRead, true);
                    if (ltr.Name == layer_name) { return true; }
                }
            }
            catch (System.Exception ex)
            {
                //Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
                ed.WriteMessage("layerExists() " + ex.ToString());
            }
            finally
            {
                trans.Dispose();
            }
            return false;
        }

        ObjectId CreateNewLayer(string layer_name)
        {
            Database db = HostApplicationServices.WorkingDatabase;
            Transaction trans = db.TransactionManager.StartTransaction();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            try
            {
                Autodesk.AutoCAD.DatabaseServices.LayerTable layerTable;
                layerTable = (Autodesk.AutoCAD.DatabaseServices.LayerTable)trans.GetObject(db.LayerTableId, OpenMode.ForRead, true);
                foreach (ObjectId str in layerTable)
                {
                    Autodesk.AutoCAD.DatabaseServices.LayerTableRecord ltr = (LayerTableRecord)trans.GetObject(str, OpenMode.ForRead, true);
                    if (ltr.Name == layer_name) { return str; }
                }
                layerTable = null;
                layerTable=(Autodesk.AutoCAD.DatabaseServices.LayerTable)trans.GetObject(db.LayerTableId,OpenMode.ForWrite,true);
                Autodesk.AutoCAD.DatabaseServices.LayerTableRecord layerTableR=new LayerTableRecord();
                layerTableR.Name=layer_name;
                layerTableR.Description = "Ведомость площадей и длин";
                ObjectId layerid= layerTable.Add(layerTableR);
                trans.AddNewlyCreatedDBObject(layerTableR, true);
                trans.Commit();
                return layerid;
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("CreateNewLayer() " + ex.ToString());
            }
            finally
            {
                trans.Dispose();
            }
            return new ObjectId();            
        }

/*      private Dictionary<string,ObjectId> GetLinetypeList()
        {
            Dictionary<string, ObjectId> linetypes = new Dictionary<string, ObjectId>();
            Document doc = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                using (trans)
                {
                    LinetypeTable lt = (LinetypeTable)trans.GetObject(db.LinetypeTableId, OpenMode.ForRead);
                    foreach (ObjectId lid in lt)
                    {
                        LinetypeTableRecord ltr = (LinetypeTableRecord)trans.GetObject(lid, OpenMode.ForRead);
                        linetypes.Add(ltr.Name,ltr.ObjectId);
                    }
                }
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("GetLinetypeList() " + ex.ToString());
            }
            finally
            {
                trans.Dispose();
            }
            return linetypes;
        }
*/

        private void RbPlineEnt_CheckedChanged(object sender, EventArgs e)
        {
            if (RbPlineEnt.Checked) { this.Text = "Подсчет суммы длин и площадей полилиний"; groupBox2.Text = "Выбор в соответствии с параметрами полилинии:"; }
            else { this.Text = "Подсчет суммы длин и площадей примитивов (Arc,Circle,Line,Polyline)"; groupBox2.Text = "Выбор в соответствии с параметрами примитивов:"; }
        }
    }
    

    class Param
    {
        public enum calctype { pline, all};
        public string DisplayMember
        {
            get
            {
                string res = "-> \"" + _descript + "\":  цвет (";
                if (_color != null) { res += _color.ColorNameForDisplay; }
                else { res += "Любой";}
                res+="), слой (" + _layer + "), тип линии (" + _linetype+")";
                return res;
            }
        }

        public Param(string descript, Autodesk.AutoCAD.Colors.Color color, string layer, string linetype, calctype type)
        {
            _descript = descript;
            _color = color;
            _layer = layer;
            _linetype = linetype;
            _type = type;
        }

        public bool Equal(Param ob)
        {
            if ((ob._layer == this._layer) && (ob._linetype == this._linetype) && (ob._color==this._color))
            { return true; }
            else
            { return false; }
        }

        public string Description
        {
            get { return _descript; }
            set { _descript = value; }
        }

        [Browsable(false)]
        public Autodesk.AutoCAD.Colors.Color Color
        {
            get { return _color; }
            set { _color = value; }
        }

        public string Layer
        {
            get { return _layer; }
            set { _layer = value; }
        }

        public string Linetype
        {
            get { return _linetype; }
            set { _linetype = value; }
        }
        public calctype Type
        {
            get { return _type; }
            set { _type = value; }
        }

        private string _descript;
        private Autodesk.AutoCAD.Colors.Color _color;
        private string _layer;
        private string _linetype;
        private calctype _type;
    }
}
