﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using System.Windows.Media.Imaging;
using System.Reflection;
using System.IO;


using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.Geometry;

using Autodesk.AutoCAD.Windows;
using Autodesk.Windows;
using Autodesk.AutoCAD;
using Autodesk.AutoCAD.Customization;

[assembly: ExtensionApplication(typeof(lufs_genplan.InitClass))]

namespace lufs_genplan
{
    public class Command_class
    {
        [CommandMethod("LUFS", "LUFS_about", CommandFlags.Transparent)]
        public void LUFS_about()
        {
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
            ed.WriteMessage("Автоматизация расчета объемов работ генплана.\n");
            ed.WriteMessage("(c) 2009. Луферов Александр Н.\n");
            ed.WriteMessage("Доступные команды:\n");
        }

        [CommandMethod("LUFS", "LUFS_uninstall", CommandFlags.Transparent)]
        public void LUFS_uninstall()
        {
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
            ed.WriteMessage("Автоматизация расчета объемов работ генплана.\n");
            ed.WriteMessage("(c) 2009. Луферов Александр Н.\n");
            ed.WriteMessage("Удаление регистрационной информации:...\n");
            InitClass init = new InitClass();
            init.Uninstall();
        }
       
        //Подсчет суммы длин и площадей полилиний отвечающим заданным условиям: слой, цвет, тип линии
        //Выбор соответствия осуществляется в форме FormLen
        [CommandMethod("LUFS", "LUFS_plinecalc", CommandFlags.Modal)]
        public void LUFS_plinecalc()
        {
            FormLen form = new FormLen();
            List<string> layers = GetLayersList();
            form.CombobLayerUse.Items.AddRange(layers.ToArray());
            List<string> linetypes = GetLinetypeList();
            form.CombobLinetUse.Items.AddRange(linetypes.ToArray());
            Application.ShowModalDialog(form);
        }

        //Подсчет суммы длин и площадей дорожной разметки (по СТБ 1033) с выводом в ексель (ТОДО или опенофис?)
        //Разметка выбирается по типам линии (заранее предопределенным) и слою на котором находится
        [CommandMethod("LUFS", "LUFS_razmcalc", CommandFlags.Modal)]
        public void LUFS_razmcalc()
        {
            FormRazm form = new FormRazm();
            List<string> layers = GetLayersList();
            form.CombobLayerUse.Items.AddRange(layers.ToArray());
            form.CombobLayerUse.Text = (string)form.CombobLayerUse.Items[0];
            Application.ShowModalDialog(form);
        }

        //Возвращает лист имен слоев
        private List<string> GetLayersList()
        {
            List<string> layers = new List<string>();
            Database db = HostApplicationServices.WorkingDatabase;
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                LayerTable lt = (LayerTable)trans.GetObject(db.LayerTableId, OpenMode.ForRead);
                
                foreach (ObjectId lid in lt)
                {
                    LayerTableRecord ltr = (LayerTableRecord)trans.GetObject(lid, OpenMode.ForRead);
                    layers.Add(ltr.Name);
                }
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("GetLayersList() " + ex.ToString());
            }
            finally
            {
                trans.Dispose();
            }
            return layers;
        }
        //return list of linetypes name
        private List<string> GetLinetypeList()
        {
            List<string> linetypes = new List<string>();
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Database db = doc.Database;
            Editor ed = doc.Editor;
            Transaction trans = db.TransactionManager.StartTransaction();
            try
            {
                using (trans)
                {
                    LinetypeTable lt = (LinetypeTable)trans.GetObject(db.LinetypeTableId, OpenMode.ForRead);
                    foreach (ObjectId lid in lt)
                    {
                        LinetypeTableRecord ltr = (LinetypeTableRecord)trans.GetObject(lid, OpenMode.ForRead);
                        linetypes.Add(ltr.Name);
                    }
                }
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("GetLinetypeList() " + ex.ToString());
            }
            finally
            {
                trans.Dispose();
            }
            return linetypes;
        }
    }

    public class LUFS_Utils
    {
        //Преобразует строку вида 1:500 в число 0.5, 1:1000 -> 1, 1:100 -> 0.1  и тп
        public static double mashtToDouble(string mashtab)
        {
            string temp = mashtab.Substring(mashtab.IndexOf(':')+1);
            return (System.Convert.ToDouble(temp)/1000D);
        }
    }
    
    class InitClass : IExtensionApplication
    {
        //ContextMenuExtension m_ContextMenu;

        const string sectionName = "LUFS_genplan";
        const string DESCRIPTION = "(C) Luferov Aleksandr 2010";
        const int LOADCTRLS = 2;
        const int MANAGED = 1;

        const string toolbarID = "LUFS_GENPLAN_TOOLBAR";

        const string pcalcText =      "Запуск LUFS_plinecalc";
        const string pcalcDesc =      "Расчет по заданным параметрам площадей и длинн объектов автокада.";
        const string pcalcSmallFile = "icon_plinecalc-16x16.ico";
        const string pcalcLargeFile = "icon_plinecalc-32x32.ico";
        const string pcalcCmd =       "_LUFS_plinecalc";

        const string rcalcText =      "Запуск LUFS_razmcalc";
        const string rcalcDesc =      "Расчет площадей и длинн дорожной разметки.";
        const string rcalcSmallFile = "icon_rlinecalc-16x16.ico";
        const string rcalcLargeFile = "icon_rlinecalc-32x32.ico";
        const string rcalcCmd =       "_LUFS_razmcalc";

        const string uninstallText = "Удаление Генплан";
        const string uninstallDesc = "Удаление всей регистрационной информации. После перезапуска автокада приложение можно удалить.";
        const string uninstallSmallFile = "icon_uninstall-16x16.ico";
        const string uninstallLargeFile = "icon_uninstall-32x32.ico";
        const string uninstallCmd = "_LUFS_uninstall";

        public void Initialize()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            if (this.isRegister() == false)
            {
                ed.WriteMessage("Приложение не активировано. Выполняется активация...\n");
                if (this.Register() == true) { ed.WriteMessage("Приложение Lufs_genplan успешно активировано (C) 2010 Aleksandr Luferov\n"); }
                else { ed.WriteMessage("Приложение Lufs_genplan НЕ активировано (C) 2010 Aleksandr Luferov\n"); }
            }
            else { ed.WriteMessage("Приложение активировано.\n"); }
            ed.WriteMessage("Lufs_genplan loaded (C) 2010 Aleksandr Luferov\n");
            //AddContextMenu();
            BuildMenuCUI();
            // We defer the creation of our Application Menu to when
            // the menu is next accessed
            //ComponentManager.ApplicationMenu.Opening += new EventHandler<EventArgs>(ApplicationMenu_Opening);
            //Application.Idle += new EventHandler(Application_OnIdle);
        }

        public void Terminate()
        {
            // Assuming these events have fired, they have already
            // been removed
            //ComponentManager.ApplicationMenu.Opening -= new EventHandler<EventArgs>(ApplicationMenu_Opening);
            //Application.Idle -= new EventHandler(Application_OnIdle);
        }

        //void Application_OnIdle(object sender, EventArgs e)
        //{
            // Remove the event when it is fired
        //    Application.Idle -= new EventHandler(Application_OnIdle);
            // Add our Quick Access Toolbar item
            //AddQuickAccessToolbarItem();
        //    BuildMenuCUI();
        //}

        /*void ApplicationMenu_Opening(object sender, EventArgs e)
        {
            // Remove the event when it is fired
            ComponentManager.ApplicationMenu.Opening -= new EventHandler<EventArgs>(ApplicationMenu_Opening);
            // Add our Application Menu
            //AddApplicationMenu();
        }*/
        /*
        private void AddApplicationMenu()
        {
            ApplicationMenu menu = ComponentManager.ApplicationMenu;
            if (menu != null && menu.MenuContent != null)
            {
                // Create our Application Menu Item
                ApplicationMenuItem mi_p = new ApplicationMenuItem();
                mi_p.Text = pcalcText;
                mi_p.Description = pcalcDesc;
                mi_p.LargeImage = LoadBMP(pcalcLargeFile);
                // Attach the handler to fire out command
                mi_p.CommandHandler = new AutoCADCommandHandler(pcalcCmd);
                // Add it to the menu conten
                mi_p.Id = pcalcCmd+"AM";
                menu.MenuContent.Items.Add(mi_p);

                ApplicationMenuItem mi_r = new ApplicationMenuItem();
                mi_r.Text = rcalcText;
                mi_r.Description = rcalcDesc;
                mi_r.LargeImage = LoadBMP(rcalcLargeFile);
                // Attach the handler to fire out command
                mi_r.CommandHandler = new AutoCADCommandHandler(rcalcCmd);
                // Add it to the menu content
                mi_r.Id = rcalcCmd+"AM";
                menu.MenuContent.Items.Add(mi_r);
            }
        }
        */
        /*
        private void AddQuickAccessToolbarItem()
        {
            Autodesk.Windows.ToolBars.QuickAccessToolBarSource qat = ComponentManager.QuickAccessToolBar;
            if (qat != null)
            {
                // Create our Ribbon Button
                RibbonButton rb_p = new RibbonButton();
                rb_p.Text = pcalcText;
                rb_p.Description = pcalcDesc;
                rb_p.Image = LoadBMP(pcalcSmallFile);
                // Attach the handler to fire out command
                rb_p.CommandHandler = new AutoCADCommandHandler(pcalcCmd);
                // Add it to the Quick Access Toolbar
                rb_p.Id = pcalcCmd + "QA";
                qat.AddStandardItem(rb_p);

                RibbonButton rb_r = new RibbonButton();
                rb_r.Text = rcalcText;
                rb_r.Description = rcalcDesc;
                rb_r.Image = LoadBMP(rcalcSmallFile);
                // Attach the handler to fire out command
                rb_r.CommandHandler = new AutoCADCommandHandler(rcalcCmd);
                // Add it to the Quick Access Toolbar
                rb_r.Id = rcalcCmd + "QA";
                qat.AddStandardItem(rb_r);
            }
        }
        */
  
        public void BuildMenuCUI()
        {
            string path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            string myCui = "lufs_genplan.cuix";
            string myCuiFile = path + "\\" + myCui;
            string myCuiFileToSend = path + "/lufs_genplan.cuix";
            const string myCuiSectionName = "LUFS_GENPLAN";

            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
            string mainCui = Application.GetSystemVariable("MENUNAME") + ".cuix";
           
            CustomizationSection cs = new CustomizationSection(mainCui);
            
            PartialCuiFileCollection pcfc = cs.PartialCuiFiles;
            if (cs.PartialCuiFiles.Contains(myCui))
            {
                //ed.WriteMessage("\nCustomization file \"" + myCui + "\" already loaded. Remove it\n");
                UnLoadMyCui(myCuiSectionName);
            }
            if (System.IO.File.Exists(myCuiFile))
            {
                //ed.WriteMessage("\nCustomization file \"" + myCuiFile + "\" exists - delete it.\n");
                System.IO.File.Delete(myCuiFile);
            }   
            
            //ed.WriteMessage("\nCustomization file \"" + myCuiFile + "\" does not exist - building it.\n");
            // Create a customization section for our partial menu
            CustomizationSection pcs = new CustomizationSection();
            pcs.MenuGroupName = myCuiSectionName;

            // Let's add a menu group, with two commands
            MacroGroup mg = new MacroGroup(myCuiSectionName, pcs.MenuGroup);
            MenuMacro mm1 = new MenuMacro(mg, "LUFS_plinecalc", "^C^CLUFS_plinecalc", "ID_LUFS_plinecalc");
            MenuMacro mm2 = new MenuMacro(mg, "LUFS_razmcalc", "^C^CLUFS_razmcalc", "ID_LUFS_razmcalc");
            MenuMacro mm3 = new MenuMacro(mg, "LUFS_Uninstall", "^C^CLUFS_uninstall", "ID_LUFS_uninstall");
            
            mm1.macro.SmallImage = path + "\\" + pcalcSmallFile;
            mm1.macro.LargeImage = path + "\\" + pcalcLargeFile;

            mm2.macro.SmallImage = path + "\\" + rcalcSmallFile;
            mm2.macro.LargeImage = path + "\\" + rcalcLargeFile;

            mm3.macro.SmallImage = path + "\\" + uninstallSmallFile;
            mm3.macro.LargeImage = path + "\\" + uninstallLargeFile;

            Toolbar tb = new Toolbar("Genplan", pcs.MenuGroup);
            tb.ToolbarOrient = ToolbarOrient.floating;
            tb.ToolbarVisible = ToolbarVisible.show;

            ToolbarItemBase tbi1 = new ToolbarButton("ID_LUFS_plinecalc", "Подсчет площадей", tb, 0);
            ToolbarItemBase tbi2 = new ToolbarButton("ID_LUFS_razmcalc", "Подсчет разметки", tb, 1);

            ToolbarItemBase tbi3 = new ToolbarButton("ID_LUFS_uninstall", "Удалить приложение", tb, 2);
            
            tb.ToolbarItems.Add(tbi1);
            tb.ToolbarItems.Add(tbi2);
            tb.ToolbarItems.Add(tbi3);

            //Finally we save the file and load it
            //ed.WriteMessage("\nSave file <" + myCuiFile + ">\n");
            pcs.SaveAs(myCuiFile);
            //ed.WriteMessage("\nLoad file <" + myCuiFileToSend + ">\n");
            LoadMyCui(myCuiFileToSend);
        }

        private void LoadMyCui(string cuiFile)
        {

            // This load technique sends a LISP string to the
            // command line (which avoid us having to set FILEDIA
            // to 0) after setting CMDECHO to 0, to minimize
            // what's displayed.
            // We make sure the LISP string resets the  value of
            // CMDECHO at the end (the string is executed
            // asynchronously, so we don't have the chance to do
            // it in our calling function).
            
            Document doc = Application.DocumentManager.MdiActiveDocument;
            
            object oldCmdEcho = Application.GetSystemVariable("CMDECHO");
            Application.SetSystemVariable("CMDECHO", 0);
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
            ed.WriteMessage("\nLoad file <" + cuiFile + ">\n");
            doc.SendStringToExecute("(command \"_.CUILOAD\" \"" + cuiFile + "\")(setvar \"CMDECHO\" " + oldCmdEcho + ")(princ) ", false, false, false);

        }

        private void UnLoadMyCui(string cuiGroup)
        {

            // This load technique sends a LISP string to the
            // command line (which avoid us having to set FILEDIA
            // to 0) after setting CMDECHO to 0, to minimize
            // what's displayed.
            // We make sure the LISP string resets the  value of
            // CMDECHO at the end (the string is executed
            // asynchronously, so we don't have the chance to do
            // it in our calling function).

            Document doc = Application.DocumentManager.MdiActiveDocument;

            object oldCmdEcho = Application.GetSystemVariable("CMDECHO");
            Application.SetSystemVariable("CMDECHO", 0);
            doc.SendStringToExecute("(command \"_.CUIUNLOAD\" \"" + cuiGroup + "\")(setvar \"CMDECHO\" " + oldCmdEcho + ")(princ) ", false, false, false);
        }

        /*private System.Windows.Media.ImageSource LoadBMP(string ico)
        {
            // We'll look for our icons in the folder of the assembly
            // (we could also use a resources, of course)
            string path = Path.GetDirectoryName( Assembly.GetExecutingAssembly().Location );
            // Check our .ico file exists
            string fileName = path + "\\" + ico;
            if (File.Exists(fileName))
            {
                // Get access to it via a stream
                Stream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                using (fs)
                {
                    // Decode the contents and return them
                    //BmpBitmapDecoder dec_b = new BmpBitmapDecoder(fs, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
                    IconBitmapDecoder dec = new IconBitmapDecoder(fs, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
                    return dec.Frames[0];
                }
            }
            else { }
            return null;
        }*/

        private bool isRegister()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            try
            {
                IConfigurationSection con = Application.UserConfigurationManager.OpenGlobalSection();
                IConfigurationSection appsec = con.OpenSubsection("Applications");
                if (appsec.ContainsSubsection(sectionName))
                {
                    System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
                    string LOADER = asm.Location;

                    Microsoft.Win32.RegistryKey autoc = (Microsoft.Win32.Registry.CurrentUser);
                    Microsoft.Win32.RegistryKey ack = autoc.OpenSubKey(HostApplicationServices.Current.RegistryProductRootKey);
                    Microsoft.Win32.RegistryKey appk = ack.OpenSubKey("Applications", true);
                    Microsoft.Win32.RegistryKey cur_app = appk.OpenSubKey(sectionName,true);
                    if (cur_app.GetValue("LOADER") == LOADER) { return true; }
                    else { 
                        cur_app.SetValue("LOADER", LOADER);
                        ed.WriteMessage("Путь загрузки приложения изменен: " + LOADER);
                    }
                    return true;
                }
            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("isRegister()\n " + ex.ToString()); 
                return false;
            }
            return false;
        }

        private bool Register()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
            string LOADER = asm.Location;
            try
            {
                Microsoft.Win32.RegistryKey autoc = (Microsoft.Win32.Registry.CurrentUser);
                Microsoft.Win32.RegistryKey ack = autoc.OpenSubKey(HostApplicationServices.Current.RegistryProductRootKey);
                Microsoft.Win32.RegistryKey appk = ack.OpenSubKey("Applications", true);
                if (appk == null) { appk = ack.CreateSubKey("Applications"); }
                Microsoft.Win32.RegistryKey cur_app = appk.CreateSubKey(sectionName);
                cur_app.SetValue("DESCRIPTION", DESCRIPTION);
                cur_app.SetValue("LOADCTRLS", LOADCTRLS);
                cur_app.SetValue("MANAGED", MANAGED);
                cur_app.SetValue("LOADER", LOADER);
            }
            catch (System.Exception ex)
            { 
                ed.WriteMessage("Register() " + ex.ToString()); 
                return false;
            }
            finally
            {}
            return true;
        }
        /*
         private void AddContextMenu()
        {
            m_ContextMenu = new ContextMenuExtension();
            m_ContextMenu.Title = "Генплан";
            Autodesk.AutoCAD.Windows.MenuItem vol_calc;
            vol_calc = new Autodesk.AutoCAD.Windows.MenuItem("Подсчет объемов");
            vol_calc.Click += new EventHandler(CallbackOnClickGenCalc);
            m_ContextMenu.MenuItems.Add(vol_calc);

            Autodesk.AutoCAD.Windows.MenuItem razm_calc;
            razm_calc = new Autodesk.AutoCAD.Windows.MenuItem("Подсчет разметки");
            razm_calc.Click += new EventHandler(CallbackOnClickRazmCalc);
            m_ContextMenu.MenuItems.Add(razm_calc);

            Autodesk.AutoCAD.Windows.MenuItem uninstal;
            uninstal = new Autodesk.AutoCAD.Windows.MenuItem("Удалить");
            uninstal.Click += new EventHandler(CallbackOnClickUninstall);
            m_ContextMenu.MenuItems.Add(uninstal);

            Autodesk.AutoCAD.ApplicationServices.Application.AddDefaultContextMenuExtension(m_ContextMenu);
        }
        */
        private void RemoveRegisterInfo()
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            try
            {
                Microsoft.Win32.RegistryKey autoc = (Microsoft.Win32.Registry.CurrentUser);
                Microsoft.Win32.RegistryKey ack = autoc.OpenSubKey(HostApplicationServices.Current.RegistryProductRootKey);
                Microsoft.Win32.RegistryKey appk = ack.OpenSubKey("Applications", true);
                if (appk.OpenSubKey(sectionName) == null) { return; }
                appk.DeleteSubKey(sectionName);

            }
            catch (System.Exception ex)
            {
                ed.WriteMessage("RemoveRegisterInfo() " + ex.ToString());
            }
            finally
            {}
        }
        /*
        private void RemoveContextMenu()
        {
            if (m_ContextMenu != null)
            {
                Autodesk.AutoCAD.ApplicationServices.Application.RemoveDefaultContextMenuExtension(m_ContextMenu);
                m_ContextMenu = null;
            }
        }

        private void CallbackOnClickGenCalc(object Sender, EventArgs e)
        {
            DocumentLock docLock = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.LockDocument();
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            ed.Document.SendStringToExecute("LUFS_plinecalc ", true, false, true);
            docLock.Dispose();
        }

        private void CallbackOnClickRazmCalc(object Sender, EventArgs e)
        {
            DocumentLock docLock = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.LockDocument();
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            ed.Document.SendStringToExecute("LUFS_razmcalc ", true, false, true);
            docLock.Dispose();
        }
        */

        public void Uninstall()
        {
            DocumentLock docLock = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.LockDocument();
            Document doc = Application.DocumentManager.MdiActiveDocument;
            Editor ed = doc.Editor;
            //RemoveContextMenu();
            RemoveRegisterInfo();

            string path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            string myCui = "lufs_genplan.cuix";
            string myCuiFile = path + "\\" + myCui;
            const string myCuiSectionName = "LUFS_GENPLAN";

            string mainCui = Application.GetSystemVariable("MENUNAME") + ".cuix";

            CustomizationSection cs = new CustomizationSection(mainCui);

            PartialCuiFileCollection pcfc = cs.PartialCuiFiles;
            if (cs.PartialCuiFiles.Contains(myCui))
            {
                UnLoadMyCui(myCuiSectionName);
            }
            if (System.IO.File.Exists(myCuiFile))
            {
                System.IO.File.Delete(myCuiFile);
            }   
            
            ed.WriteMessage("Приложение деактивировано.\n После перезапуска автокада можно удалить файл приложения.\n");
            docLock.Dispose();
        }
    }
}

public class AutoCADCommandHandler : System.Windows.Input.ICommand
{
    private string _command = "";
    
    public AutoCADCommandHandler(string cmd)
    {
        _command = cmd;
    }
        
#pragma warning disable 67

    public event EventHandler CanExecuteChanged;

#pragma warning restore 67
    
    public bool CanExecute(object parameter)
    {
        return true;
    }
        
    public void Execute(object parameter)
    {
        if (!String.IsNullOrEmpty(_command))
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            doc.SendStringToExecute( _command + " ", false, false, false );
        }
    }
}